from typing import List, Dict, Any, Tuple
from src.detectors.interface import Detector
from src.detections import Detection, MorphosyntacticInfoMixin
from src.annotations import Annotation, NerAnnotation, MorphosyntacticAnnotation
from src.detections import DETECTION_CLASSES_MAP


class NerDetector(Detector):
    def __init__(self, detection_mapping: Dict[str, str], language: str = "pl") -> None:
        self._language = language
        self._detection_mapping = detection_mapping

    def detect(
        self, text: str, annotations: List[Tuple[int, int, Annotation]]
    ) -> List[Tuple[int, int, str]]:

        morpho_tags = dict()
        ner_detections = []

        for annotation in annotations:
            start, end, annotation = annotation
            if isinstance(annotation, MorphosyntacticAnnotation):
                morpho_tags[start] = annotation.morphosyntactic_tag
            elif isinstance(annotation, NerAnnotation):
                ner_type = annotation.ner_type

                if ner_type in self._detection_mapping:
                    detection_class = DETECTION_CLASSES_MAP[
                        self._detection_mapping[ner_type]
                    ]
                    ner_detections.append((start, end, detection_class))

        result = []
        for start, end, ner_detection in ner_detections:
            kwargs = dict()
            if issubclass(ner_detection, MorphosyntacticInfoMixin):
                if start in morpho_tags:
                    kwargs["morpho_tag"] = morpho_tags[start]

            result.append((start, end, ner_detection(**kwargs)))

        return result
